﻿using HarmonyLib;
using System.Collections.Generic;
using System.Linq;
using System.Reflection;
using UnityEngine;

namespace Whitebrim.LeavesAsCompost
{
    public class LeavesAsCompost : Mod
    {
        private const int THATCH_FUEL_VALUE = 5;
        private const int PLANK_FUEL_VALUE = 10;
        SO_FuelValue thatchData, plankData;
        Harmony harmonyInstance;

        public void Awake()
        {
            harmonyInstance = new Harmony("com.whitebrim.leavesascompost");
            harmonyInstance.PatchAll(Assembly.GetExecutingAssembly());
            CreadeSO();
            if (RAPI.IsCurrentSceneGame())
            {
                AddFuelType();
                PatchAllCompostersManually();
            }
            Debug.Log("LeavesAsCompost has been loaded!");
        }

        private void CreadeSO()
        {
            thatchData = ScriptableObject.CreateInstance<SO_FuelValue>();
            thatchData.itemType = ItemManager.GetItemByName("Thatch");
            thatchData.fuelValueOfType = THATCH_FUEL_VALUE;
            thatchData.fuelFiltrationType = FuelValue_FiltrationType.FV_COMPOSTCOMPOSTTANK;
            thatchData.name = "FuelValue_Thatch_CompostTank";

            plankData = ScriptableObject.CreateInstance<SO_FuelValue>();
            plankData.itemType = ItemManager.GetItemByName("Plank");
            plankData.fuelValueOfType = PLANK_FUEL_VALUE;
            plankData.fuelFiltrationType = FuelValue_FiltrationType.FV_COMPOSTCOMPOSTTANK;
            plankData.name = "FuelValue_Plank_CompostTank";
        }

        public override void WorldEvent_WorldLoaded()
        {
            AddFuelType();
        }

        private void AddFuelType()
        {
            var liquidFueelManager = FindObjectOfType<LiquidFuelManager>();
            if (Traverse.Create(liquidFueelManager).Field("initialized").GetValue<bool>())
            {
                LiquidFuelManager.fuelTypesStatic = LiquidFuelManager.fuelTypesStatic
                    .AddItem(thatchData)
                    .AddItem(plankData)
                    .ToArray();
            }
            liquidFueelManager.fuelTypes = liquidFueelManager.fuelTypes
                .AddItem(thatchData)
                .AddItem(plankData)
                .ToArray();
        }

        private void PatchAllCompostersManually()
        {
            var composters = FindObjectsOfType<Placeable_BiofuelExtractor>();
            if (composters != null)
            {
                var thatch = ItemManager.GetItemByName("Thatch");
                var plank = ItemManager.GetItemByName("Plank");
                foreach (var composter in composters)
                {
                    var traverse = typeof(Tank_BiofuelExtractor).BaseType.GetField("acceptableTypes", BindingFlags.Instance | BindingFlags.NonPublic);
                    List<Item_Base> list = traverse.GetValue(composter.compostTank) as List<Item_Base>;
                    list.Add(thatch);
                    list.Add(plank);
                }
            }
        }

        public void OnModUnload()
        {
            var liquidFueelManager = FindObjectOfType<LiquidFuelManager>();
            liquidFueelManager.fuelTypes = liquidFueelManager.fuelTypes.Where(x => x != thatchData && x != plankData).ToArray();
            LiquidFuelManager.fuelTypesStatic = LiquidFuelManager.fuelTypesStatic.Where(x => x != thatchData && x != plankData).ToArray();
            var composters = FindObjectsOfType<Placeable_BiofuelExtractor>();
            if (composters != null)
            {
                var thatch = ItemManager.GetItemByName("Thatch");
                var plank = ItemManager.GetItemByName("Plank");
                foreach (var composter in composters)
                {
                    var traverse = typeof(Tank_BiofuelExtractor).BaseType.GetField("acceptableTypes", BindingFlags.Instance | BindingFlags.NonPublic);
                    List<Item_Base> list = traverse.GetValue(composter.compostTank) as List<Item_Base>;
                    list.RemoveAll((x => x == thatch || x == plank));
                }
            }

            harmonyInstance.UnpatchAll();
            Debug.Log("LeavesAsCompost has been unloaded!");
            Destroy(gameObject);
        }
    }

    class Placeable_BiofuelExtractorPatch
    {
        [HarmonyPatch(typeof(Placeable_BiofuelExtractor), "OnBlockPlaced")]
        class OnBlockPlaced
        {
            private static void Prefix(Placeable_BiofuelExtractor __instance, bool ___hasBeenPlaced)
            {
                if (!___hasBeenPlaced)
                {
                    var traverse = typeof(Tank_BiofuelExtractor).BaseType.GetField("acceptableTypes", BindingFlags.Instance | BindingFlags.NonPublic);
                    List<Item_Base> list = traverse.GetValue(__instance.compostTank) as List<Item_Base>;
                    list.Add(ItemManager.GetItemByName("Thatch"));
                    list.Add(ItemManager.GetItemByName("Plank"));
                }
            }
        }
    }
}